#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2util.h>
#include <sps2regstructs.h>
#include <assert.h>
#include <memory.h>
#include <signal.h>
#include "PS2Defines.h"
#include "ps2maths.h"
#include "ps2matrix4x4.h"
#include "pad.h"
#include "sps2wrap.h"
#include "dma.h"
#include "texture.h"

bool g_bLoop = true;

void sig_handle(int sig)
{
	g_bLoop = false;
}

int main()
{
	// Initialise control pad 0
	if(!pad_init(PAD_0, PAD_INIT_LOCK | PAD_INIT_ANALOGUE | PAD_INIT_PRESSURE))
	{
		printf("Failed to initialise control pad\n");
		pad_cleanup(PAD_0);
		exit(0);
	}	

	SPS2Manager.Initialise(4096);	// 4096 * 4K Pages = 16MB Total

	VIFStaticDMA.Initialise(1536);	// 1536 * 4K Pages = 6MB Static DMA
	VIFDynamicDMA.Initialise(256);	// 256 * 4K Pages * 2 Buffers =
									// 2MB Dynamic DMA

	// Register our signal function for every possible signal (i.e. ctrl + c)
	for(int sig=0; sig<128; sig++)
		signal(sig, sig_handle);
		
	// Set up the DMA packet to clear the screen.
	SPS2Manager.InitScreenClear(0x20, 0, 0x20);	
		
	// Load texture.
	CTexture Tex1;
	Tex1.LoadBitmap("max8bit256.bmp");

	while(g_bLoop)
	{
		// Fire off the previous frame's buffer (or init data if it's the first time)
		VIFDynamicDMA.Fire();

		pad_update(PAD_0);

		// Clear the screen
		SPS2Manager.BeginScene();
		
		// Upload the current texture to the free texture buffer
		Tex1.Upload(480);
		Tex1.Select();
		
		// Start DIRECT mode transfer
		VIFDynamicDMA.StartDirect();
		
		// Add the GIFTag
		VIFDynamicDMA.Add128(GS_GIFTAG_BATCH(	1,							// NLOOP
	 											1,							// EOP
	 											1,							// PRE
	 											GS_PRIM(PRIM_SPRITE, 		//PRIM
														PRIM_IIP_FLAT, 
														PRIM_TME_ON,
														PRIM_FGE_OFF, 
														PRIM_ABE_ON, 
														PRIM_AA1_OFF, 
														PRIM_FST_UV, 
														PRIM_CTXT_CONTEXT1, 
														PRIM_FIX_NOFIXDDA),					
	 											GIF_FLG_PACKED,				//FLG
	 											GS_BATCH_5(	GIF_REG_RGBAQ, 	//BATCH
	 														GIF_REG_UV, 
	 														GIF_REG_XYZ2, 
	 														GIF_REG_UV, 
	 														GIF_REG_XYZ2)));					
			 
 
		// Add Vertex data
		
		// Add the colour
    	VIFDynamicDMA.Add128(PACKED_RGBA(0x80, 0x80, 0x80, 0x80));
    	
    	// Add UV texture coordinates
    	VIFDynamicDMA.Add128(PACKED_UV(0<<4, 0<<4));
    	    	
    	// Add the position   	   	
    	VIFDynamicDMA.Add128(PACKED_XYZ2(	(2048 - 128)<< 4, 
    										(2048 - 128)<< 4, 
    										128, 
    										0));
    	
    	// Add UV texture coordinates									
    	VIFDynamicDMA.Add128(PACKED_UV((255<<4), (255<<4)));
    	    	
    	// Add the position   	   	
    	VIFDynamicDMA.Add128(PACKED_XYZ2(	(2048 + 128)<< 4, 
    										(2048 + 128)<< 4, 
    										128, 
    										0));
	    	  	
		
		// End the direct mode
		VIFDynamicDMA.EndDirect();
	
		// Swap the buffers
		SPS2Manager.EndScene();
		
		// Check for exit condition
		if((pad[0].buttons & PAD_START)&&(pad[0].buttons & PAD_SELECT)) g_bLoop = false;
	}
	
	// Cleanup the Pads
	pad_cleanup(PAD_0);
	
	return 0;
}

